# -*- coding: utf-8 -*-
from HTMLParser import HTMLParseError

from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib import messages
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import Q
from django.forms.widgets import PasswordInput, HiddenInput
from django.utils.translation import ugettext_lazy as _

from jmb.core.admin.options import ExtendibleModelAdmin, SettingsAdmin
from jmb.core.forms import DateWidget

from jmb.mailer.models import EmailObject, Attachment, LinkStatus
from jmb.mailer.models import MailingStatus, BounceServer
from jmb.mailer.models import SMTPServer, Link
from jmb.mailer.sender import Mailer


#####################################
#  MAILER ADMIN                      #
#####################################
class AttachmentAdminInline(admin.TabularInline):
    model = Attachment
    extra = 1
    fieldsets = ((None, {'fields': (('title', 'file_attachment'))}),)


class EmailObjectAdmin(ExtendibleModelAdmin):
#    class Media:
#        css = {"all": ("mailer/css/admin_mailer.css",)}

    date_hierarchy = 'date_create'
    #search_form = EmailObjectSearchForm
    list_display = (
        'status', 'date_create', 'email_address', 'subject', 'smtp_server',
        'bounce_server',
        'get_content_object', 'send_from_date', 'send_to_date',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'email_address', 'subject', 'content')
        }),
        (_('Dates'), {
           'fields': (
               ('send_from_date', 'send_to_date',),
           )
        }),
        (_('Configuration'), {
            'fields': (
                ('smtp_server', 'bounce_server',),
                ('header_sender', 'header_reply',),
                'slug',
            ),
            'classes': ('collapse',)
        }),
    )
    prepopulated_fields = {'slug': ('subject',)}
    inlines = (AttachmentAdminInline,)
    actions = ['send_mail_test', 'make_ready_to_send', 'make_cancel_sending']
    advanced_search_fields = (
        ('status__in',),
        ('date_create__gte', 'date_create__lte',),
        ('email_address__icontains',),
        ('subject__icontains',),
        ('send_from_date__gte', 'send_from_date__lte',),
        ('send_to_date__gte', 'send_to_date__lte',),
    )
    readonly_fields = ('send_from_date', 'send_to_date')

    def get_content_object(self, instance):
        return instance.content_object.__unicode__()

    def get_actions(self, request):
        actions = super(EmailObjectAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
            if not request.user.has_perm('emailobject.can_change_status'):
                del actions['make_ready_to_send']
                del actions['make_cancel_sending']
        except:
            pass
        return actions

#    def queryset(self, request):
#        queryset = super(EmailAdmin, self).queryset(request)
#        if not request.user.is_superuser and USE_WORKGROUPS:
#            newsletters_pk = request_workgroups_newsletters_pk(request)
#            queryset = queryset.filter(pk__in=newsletters_pk)
#        return queryset

#    def formfield_for_foreignkey(self, db_field, request, **kwargs):
#        if db_field.name == 'mailing_list' and \
#               not request.user.is_superuser and USE_WORKGROUPS:
#            mailinglists_pk = request_workgroups_mailinglists_pk(request)
#            kwargs['queryset'] = MailingList.objects.filter(pk__in=mailinglists_pk)
#            return db_field.formfield(**kwargs)
#        return super(NewsletterAdmin, self).formfield_for_foreignkey(
#            db_field, request, **kwargs)

#    def formfield_for_choice_field(self, db_field, request, **kwargs):
#        if db_field.name == 'status' and \
#               not request.user.has_perm('newsletter.can_change_status'):
#            kwargs['choices'] = ((Newsletter.DRAFT, _('Default')),)
#            return db_field.formfield(**kwargs)
#        return super(NewsletterAdmin, self).formfield_for_choice_field(
#            db_field, request, **kwargs)

#    def formfield_for_manytomany(self, db_field, request, **kwargs):
#        if db_field.name == 'test_contacts':
#            queryset = Contact.objects.filter(tester=True)
#            if not request.user.is_superuser and USE_WORKGROUPS:
#                contacts_pk = request_workgroups_contacts_pk(request)
#                queryset = queryset.filter(pk__in=contacts_pk)
#            kwargs['queryset'] = queryset
#        return super(NewsletterAdmin, self).formfield_for_manytomany(
#            db_field, request, **kwargs)

#    def save_model(self, request, newsletter, form, change):
#        workgroups = []
#        if not newsletter.pk and not request.user.is_superuser \
#               and USE_WORKGROUPS:
#            workgroups = request_workgroups(request)
#
#        if newsletter.content.startswith('http://'):
#            if CAN_USE_PREMAILER:
#                try:
#                    premailer = Premailer(newsletter.content.strip())
#                    newsletter.content = premailer.transform()
#                except PremailerError:
#                    self.message_user(request, _('Unable to download HTML, due to errors within.'))
#            else:
#                self.message_user(request, _('Please install lxml for parsing an URL.'))
#        if not request.user.has_perm('newsletter.can_change_status'):
#            newsletter.status = form.initial.get('status', Newsletter.DRAFT)
#
#        newsletter.save()
#
#        for workgroup in workgroups:
#            workgroup.newsletters.add(newsletter)

    def send_mail_test(self, request, queryset):
        """Send newsletter in test"""
        for email in queryset:
            if email.status == settings.MAILER_EMAIL_STATUS_TO_SEND_IN_TEST:
                mailer = Mailer(email, test=True, verbose=2)
                try:
                    mailer.send()
                except HTMLParseError:
                    messages.error(request, _('Unable send mail, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % email)
            else:
                messages.error(request, _('%s is not email test.') % email)

    send_mail_test.short_description = _('Send test email')

    def make_ready_to_send(self, request, queryset):
        """Make emails ready to send"""
        queryset = queryset.filter(status=settings.MAILER_EMAIL_STATUS_DRAFT)
        for email in queryset:
            email.status = settings.MAILER_EMAIL_STATUS_TO_SEND
            email.save()
        self.message_user(request, _('%s emails are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(
            Q(status=settings.MAILER_EMAIL_STATUS_TO_SEND)
        )
        for email in queryset:
            email.status = settings.MAILER_EMAIL_STATUS_CANCELED
            email.save()
        self.message_user(request, _('%s emails are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')


##############################
# BOUNCER                    #
##############################
class BounceServerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(BounceServerAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(
            widget=PasswordInput(),
            required=False
        )

    class Meta:
        model = BounceServer


class BounceServerAdmin(ExtendibleModelAdmin):
#    class Media:
#        css = {"all": ("mailer/css/admin_mailer.css",)}

    form = BounceServerAdminForm
    list_display = (
        'name', 'host', 'port', 'user', 'tls',
        'get_edit_icon', 'get_delete_icon'
    )
    no_display_links = True
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls',
        )}),
    )
    advanced_search_fields = (
        ('host', 'port',),
        ('user',),
    )
    actions = ['check_connections']

    def get_actions(self, request):
        actions = super(BounceServerAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def check_connections(self, request, queryset):
        """Check the Bouncer connection"""
        message = '%s connection %s'
        for server in queryset:
            try:
                bouncer = server.connect()
                if bouncer:
                    status = 'OK'
                    bouncer.quit()
                else:
                    status = 'KO'
            except:
                status = 'KO'
            self.message_user(request, message % (server.__unicode__(), status))
    check_connections.short_description = _('Check connection')


##############################
# SMTP                       #
##############################
class SMTPServerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(SMTPServerAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(
            widget=PasswordInput(),
            required=False
        )

    def clean_headers(self):
        """Check if the headers are well formated"""
        for line in self.cleaned_data['headers'].splitlines():
            elems = line.split(':')
            if len(elems) < 2:
                raise ValidationError(_('Invalid syntax, do not forget the ":".'))
            if len(elems) > 2:
                raise ValidationError(_('Invalid syntax, several assignments by line.'))

        return self.cleaned_data['headers']

    class Meta:
        model = SMTPServer


class SMTPServerAdmin(ExtendibleModelAdmin):
#    class Media:
#        css = {"all": ("mailer/css/admin_mailer.css",)}

    form = SMTPServerAdminForm
    list_display = (
        'name', 'host', 'port', 'user', 'tls', 'get_credits',
        'get_edit_icon', 'get_delete_icon'
    )
    #list_filter = ('tls',)
    #search_fields = ('name', 'host', 'user')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls', 'mails_hour',
        )}),
        (_('Miscellaneous'), {
            'fields': ('headers',),
            'classes': ('collapse', )
        }),
    )
    advanced_search_fields = (
        ('host', 'port',),
        ('user',),
    )
    actions = ['check_connections']

    def get_actions(self, request):
        actions = super(SMTPServerAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

#    inlines = [CreditAdminInline,]

    def check_connections(self, request, queryset):
        """Check the SMTP connection"""
        message = '%s connection %s'
        for server in queryset:
            try:
                smtp = server.connect()
                if smtp:
                    status = 'OK'
                    smtp.quit()
                else:
                    status = 'KO'
            except:
                status = 'KO'
            self.message_user(request, message % (server.__unicode__(), status))
    check_connections.short_description = _('Check connection')

#    def get_urls(self):
#        from django.conf.urls import patterns, url
#        urls = super(SMTPServerAdmin, self).get_urls()
#        my_urls = patterns('',
#            url(
#                r'^(?P<smtp_id>\d+)/set_credits/$',
#                self.admin_site.admin_view(newsletter_views.set_credits),
#                name='newsletter_smtpserver_set_credits'
#            ),
#        )
#        return my_urls + urls

#    def set_credits(self, obj):
#        return """
#            <a class="iframe" href="%(url_reverse)s?_popup=1&nobutton=1">
#                <img
#                    src='%(media_url)sadmin/images/comment.png'
#                    title=\"Set Credits\" style=\"vertical-align:middle;\"
#                />
#            </a>
#        """ % {
#            'url_reverse': reverse('admin:newsletter_smtpserver_set_credits', args=str(obj.id)),
#            'media_url': settings.MEDIA_URL,
#        }
#    set_credits.allow_tags=True


##############################
# LINK                       #
##############################
class LinkAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    #search_form = LinkSearchForm
    list_display = (
        'date_create', 'get_email', 'title', 'url',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'email_object', 'title', 'url',)
        }),
    )
    advanced_search_fields = (
        ('date_create__gte', 'date_create__lte',),
        ('email_object__email_address__icontains', 'email_object__subject__icontains'),
        ('title__icontains',),
        ('url__icontains',),
    )

    def get_actions(self, request):
        actions = super(LinkAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.email_object.email_address,
            result.email_object.subject
        )
    get_email.short_description = _("Email")


##############################
# MAILING STATUS             #
##############################
class MailingStatusAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    list_display = (
        'get_status_display', 'date_create', 'get_email_object', 'get_email_address', 'get_email_subject',
        'get_link', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'date_create', 'get_email_address', 'get_email_subject',
        'get_link_csv'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'email_object', 'rfc_status', 'diagnostic_code')
        }),
    )
    advanced_search_fields = (
        ('status__in', 'rfc_status__icontains', 'diagnostic_code__icontains'),
        ('date_create__gte', 'date_create__lte',),
        ('email_object__email_address__icontains', 'email_object__subject__icontains',),
        ('email_object__content_type__id', 'email_object__object_id'),
        ('status_link__link__title__icontains', 'status_link__link__url__icontains',),
    )

    def lookup_allowed(self, lookup, *args, **kwargs):
        return True

    def get_actions(self, request):
        actions = super(MailingStatusAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.email_object.email_address,
            result.email_object.subject
        )
    get_email.short_description = _("Email")

    def get_email_address(self, result):
        return "%s" % result.email_object.email_address
    get_email_address.short_description = _("Email Address")

    def get_email_subject(self, result):
        return "%s" % result.email_object.subject
    get_email_subject.short_description = _("Email Subject")

    def get_email_object(self, result):
        return "%s" % result.email_object.content_object
    get_email_object.short_description = _("Object")

    def get_link(self, result):
        field = ""
        if result.status_link:
            title = result.status_link.link.title
            if not title:
                title = result.status_link.link.url

            field = """
                <a href="%s" title="%s" target="_blank">%s</a>
            """ % (
                result.status_link.link.url,
                title, title
            )
        return field
    get_link.short_description = _("Link")
    get_link.allow_tags = True

    def get_link_csv(self, result):
        try:
            return result.status_link.link.title
        except:
            return ""

    def get_status(self, result):
        if result.rfc_status:
            field = """
                <span title="%s: %s">%s</span>
            """ % (
                result.rfc_status,
                result.diagnostic_code or "",
                result.get_status_display()
            )
        else:
            field = """
                <span title="%s">%s</span>
            """ % (
                result.diagnostic_code or "",
                result.get_status_display()
            )
        return field        
    get_status.short_description = _("Status")
    get_status.allow_tags = True


##############################
# LINK STATUS                #
##############################
class LinkStatusAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    list_display = (
        'date_create', 'get_email', 'get_link',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'mailing_status',
            'link',
        )}),
    )
    advanced_search_fields = (
        ('date_create__gte', 'date_create__lte',),
        ('mailing_status__email_object__email_address__icontains', 'mailing_status__email_object__subject__icontains',),
        ('link__title__icontains', 'link__url__icontains'),
    )

    def get_actions(self, request):
        actions = super(LinkStatusAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.mailing_status.email_object.email_address,
            result.mailing_status.email_object.subject
        )
    get_email.short_description = _("Email")

    def get_link(self, result):
        return "%s: %s" % (
            result.link.title,
            result.link.url
        )
    get_link.short_description = _("Link")

#admin.site.register(Settings)
admin.site.register(EmailObject, EmailObjectAdmin)
admin.site.register(SMTPServer, SMTPServerAdmin)
admin.site.register(BounceServer, BounceServerAdmin)
admin.site.register(LinkStatus, LinkStatusAdmin)
admin.site.register(Link, LinkAdmin)
admin.site.register(MailingStatus, MailingStatusAdmin)