# -*- coding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _

import admin_tools
from admin_tools.dashboard import modules, register, AppIndexDashboard
from jmb.core.admin import dashboard as jmb_dashboard

class DefaultDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Mailers",
            'display': "tabs",
            'children': [
                MailerModels(),
                MailerOtherModels(),
                ]
        }
        super(DefaultDashboard, self).__init__(**kwargs)

class MailerDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Mailers",
            'display': "tabs",
            'children': [
                MailerModels(),
                MailerOtherModels(),
                ]
        }
        super(MailerDashboard, self).__init__(**kwargs)

class MailerModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Models"),
            'models': ('mailer.models.EmailObject', 'mailer.models.MailingStatus'),
            }
        super(MailerModels, self).__init__(**kwargs)

class MailerOtherModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Others"),
            'models': (
                'mailer.models.SMTPServer', 'mailer.models.BounceServer',
                'mailer.models.Link'
            ),
        }
        super(MailerOtherModels, self).__init__(**kwargs)

class DefaultAppDashboard(AppIndexDashboard):
    def __init__(self, app_title, models, **kwargs):
        AppIndexDashboard.__init__(self, app_title, models, **kwargs)
        self.columns = 3

    def init_with_context(self, context):
        request = context['request']
        self.children.append(
            DefaultDashboard(),
        )

# Used in /admin
jmb_dashboard.register_group(MailerDashboard, 'jmb.mailer')
# Used in /admin/Organization
admin_tools.dashboard.register(DefaultAppDashboard, 'jmb.mailer')
