# coding: utf-8
"""Command for sending the newsletter"""

import sys
import logging

from optparse import make_option

import datetime
from django.conf import settings
from django.utils.translation import activate
from django.core.management.base import BaseCommand

from jmb.mailer.sender import Mailer
from jmb.mailer.models import EmailObject

log = logging.getLogger("mailer")


class Command(BaseCommand):
    """Send the newsletter in queue"""
    help = 'Send mails in queue'

    option_list = BaseCommand.option_list + (
        make_option('-c', '--set-canceled', action='store_true', default=False,
                    dest='set_canceled', help='Set all emails in error when creating Mailer object, as cancelled'),
        )
    
    def handle(self, **options):
        verbose = int(options['verbosity'])
        now = datetime.datetime.now()

        log.debug('Starting sending emails...')

        activate(settings.LANGUAGE_CODE)

        email_objects = EmailObject.objects.exclude(
            status=settings.MAILER_EMAIL_STATUS_SENT
        ).exclude(
            status=settings.MAILER_EMAIL_STATUS_DRAFT
        ).exclude(
            status=settings.MAILER_EMAIL_STATUS_CANCELED
        ).filter(
            status=settings.MAILER_EMAIL_STATUS_TO_SEND
        )

        if not email_objects:
            log.debug("empty query email_objects")
            return

        for email in email_objects:

            if email.smtp_server.get_credits() <= 0:
                log.debug("smtp credit is over")
                return

            if email.send_from_date and email.send_to_date:
                if not email.send_from_date <= now <= email.send_to_date:
                    continue

            try:
                generic_object = email.content_object
                mailer = Mailer(generic_object, verbose=verbose)
                log.debug('instanza Mailer per %s: %s' % (email.email_address, email.subject))
                log.debug('Start emailing %s: %s' % (email.email_address, email.subject))
                mailer.send(email)
                ## FIXME: non è vero! mailer.send non spedisce sempre
                #log.debug('%s: %s sent' % (email.email_address, email.subject))

                if hasattr(generic_object, "sent"):
                    generic_object.sent = True
                    generic_object.save()

            except Exception, e:
                log.debug('Problem emailing %s: %s - %s' % (email.email_address, email.subject, e))
                if options.get('set_canceled'):
                    email.status = settings.MAILER_EMAIL_STATUS_CANCELED
                    email.save()
                    log.debug('%s: %s canceled' % (email.email_address, email.subject))

        log.debug('End session sending')
