"""Mailer for send email"""
import datetime
import email
import traceback
from html2text import html2text as html2text_orig
import icalendar
import mimetypes
from random import sample
import re
import logging
from StringIO import StringIO
from smtplib import SMTPRecipientsRefused


try:
    from email.mime.multipart import MIMEMultipart
    from email.mime.text import MIMEText
    from email.mime.Encoders import encode_base64
    from email.mime.MIMEAudio import MIMEAudio
    from email.mime.MIMEBase import MIMEBase
    from email.mime.MIMEImage import MIMEImage
except ImportError:  # Python 2.4 compatibility
    from email.MIMEMultipart import MIMEMultipart
    from email.MIMEText import MIMEText
    from email.Encoders import encode_base64
    from email.MIMEAudio import MIMEAudio
    from email.MIMEBase import MIMEBase
    from email.MIMEImage import MIMEImage


from email import message_from_file
from email.utils import formatdate
from html2text import html2text as html2text_orig
from icalendar import Event as iEvent

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.http import HttpRequest
from django.template import Context, Template
from django.template.loader import render_to_string, get_template
from django.utils.encoding import smart_str
from django.utils.encoding import smart_unicode
from django.db import transaction, IntegrityError, DatabaseError
from django.template import loader
from django.template.context import RequestContext
from django.utils.http import int_to_base36
from django.utils.translation import ugettext_lazy as _

from jmb.core.middleware.thread_local import get_request
from jmb.mailer.models import MailingStatus
from jmb.mailer.utils.tokens import tokenize
from jmb.mailer.utils.mail_utils import track_links
from jmb.mailer.utils.mail_utils import body_insertion
from jmb.mailer.models import EmailObject, SMTPServer, BounceServer


LINK_RE = re.compile(r"https?://([^ \n]+\n)+[^ \n]+", re.MULTILINE)
log = logging.getLogger("mailer")


def html2text(html):
    """Use html2text but repair newlines cutting urls.
    Need to use this hack until
    https://github.com/aaronsw/html2text/issues/#issue/7 is not fixed"""
    txt = html2text_orig(html)
    links = list(LINK_RE.finditer(txt))
    out = StringIO()
    pos = 0
    for l in links:
        out.write(txt[pos:l.start()])
        out.write(l.group().replace('\n', ''))
        pos = l.end()
    out.write(txt[pos:])
    return out.getvalue()


class Mailer(object):
    """Mailer for generating and sending email
    In test mode the mailer always send mails but do not log it"""

    def __init__(
            self, generic_object, test=False, verbose=2, is_newsletter=False,
            is_newsletter2=False, is_task=False, request=None, template=None, is_trace_request=False, is_reminder=False,
            is_confirm=False,
            test_in_subject=True,
            subject=None, email_from=None
    ):
        self.generic_object = generic_object
        self.test = test
        self.verbose = verbose
        self.is_newsletter = is_newsletter
        self.is_newsletter2 = is_newsletter2
        self.is_task = is_task
        self.request = request
        self.test_in_subject = test_in_subject
        self.subject = subject
        self.template = template or self.generic_object.get_default_template_path()
        if is_reminder:
            self.template = self.generic_object.get_default_reminder_template_path()
        elif is_confirm:
            self.template = self.generic_object.get_default_confirm_template_path()
        self.is_trace_request = is_trace_request
        self.email_from = email_from


    def _get_newsletter_html_output(self, context_add):
        """
        generate newsletter HTML output
        """
        if not self.is_task and self.request:
            self.request = get_request()
        elif not self.request:
            self.request = HttpRequest() 

        context = RequestContext(self.request)
        for key, val in context_add.iteritems():
            context[key] = val

        if self.is_newsletter:
            template = Template(self.generic_object.content)
            return template.render(context)

        # TODO: Verificare funzionamento per renderizzazione eventi
        return_string_template = loader.render_to_string(self.template, context)
        if self.is_newsletter2:
            template = loader.get_template_from_string(return_string_template)
            return_string_template = template.render(context)

        return return_string_template

    def create_message_from_email(self, string_email, context_add):
        try:
            email_object = EmailObject()
            if self.test:
                """ lo stato dell'email e' 'sent test' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND_IN_TEST
            else:
                """ lo stato dell'email e' 'to send' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND

            subject = self.subject or self.generic_object.get_default_title()
            if self.test and self.test_in_subject:
                subject += " (TEST)"

            email_object.email_address = string_email
            email_object.subject = subject
            email_object.content_object = self.generic_object
            email_object.content = self._get_newsletter_html_output(context_add)
            email_object.slug = datetime.datetime.now().strftime("%Y%m%d_%H%M%S%f")
            email_object.smtp_server = SMTPServer.objects.get()
            email_object.bounce_server = BounceServer.objects.get()
            if self.email_from:
                email_object.header_sender = self.email_from
            email_object.save()
            return True, email_object
        except Exception, e:
            return False, e

    def create_message(self, contact, context_add, check_can_create=True, date_from=None, date_to=None):

        self.contact = contact
        if check_can_create:

            if not self.is_trace_request:
                if not self.contact.can_create_message[0]:
                    return self.contact.can_create_message
            else:
                if not self.contact.can_create_message_no_bonifica[0]:
                    return self.contact.can_create_message_no_bonifica

        try:
            email_object = EmailObject()
            if self.test:
                """ lo stato dell'email e' 'sent test' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND_IN_TEST
            else:
                """ lo stato dell'email e' 'to send' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND

            contact_email = self.contact.get_default_email()
            if not contact_email:
                return False, _("set an active and default email")
            subject = self.subject or self.generic_object.get_default_title()
            email_object.email_address = contact_email

            email_object.subject = subject
            if self.test and self.test_in_subject:
                email_object.subject += " (TEST)"

            email_object.content_object = self.generic_object
            email_object.content = self._get_newsletter_html_output(context_add)
            email_object.slug = "%s_%s" % (
                datetime.datetime.now().strftime("%Y%m%d_%H%M%s_%f"),
                self.contact.id
            )
            email_object.smtp_server = SMTPServer.objects.get()
            email_object.bounce_server = BounceServer.objects.get()
            if self.email_from:
                email_object.header_sender = self.email_from
            email_object.send_from_date = date_from if isinstance(date_from, datetime.datetime) else None
            email_object.send_to_date = date_to if isinstance(date_to, datetime.datetime) else None
            email_object.save()
            return True, email_object

        except Exception as e:
            log.debug(traceback.format_exc())
            return False, e


    @transaction.commit_manually
    def send(self, email_object):
        self.email = email_object

        """Send the mail"""
        if not self.can_send:
            transaction.rollback()
            log.debug("Abort due to can_send returned False")
            return

        try:
            self.smtp_connect()

            log.debug('%s email will be sent' % self.email.email_address)

            self.attachments = self.build_attachments()

            # creo status messaggio
            if self.test:
                # imposto stati di test
                to_send_status = settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND_IN_TEST
                sent_status = settings.MAILER_MAILINGSTATUS_STATUS_SENT_TEST
            else:
                # imposto stati di invio
                to_send_status = settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND
                sent_status = settings.MAILER_MAILINGSTATUS_STATUS_SENT
            try:
                mail_status = MailingStatus(
                    email_object=self.email,
                    status=to_send_status
                )
                mail_status.save()
                # Setto stato dell'EMAIL: TO_SEND or TO_SEND_IN_TEST
                self.email.set_status(mail_status)
                transaction.commit()
                log.debug("Email to send")
            except Exception, e:
                transaction.rollback()
                log.debug("Error Email to send:", e)
                return

            message = self.build_message()

            try:

                from_addr = self.email.bounce_server.user
                to_addr = self.email.email_address
                msg = message.as_string()
                self.smtp.sendmail(
                    from_addr,
                    to_addr,
                    msg
                )

                mail_status = MailingStatus(
                    email_object=self.email,
                    status=sent_status
                )
                mail_status.save()
                # Setto stato dell'EMAIL: SENT_TEST or SENT
                self.email.set_status(mail_status)
                transaction.commit()
                log.debug("Email sent")
            except SMTPRecipientsRefused, e:
                mail_status = MailingStatus(
                    email_object=self.email,
                    status=settings.MAILER_MAILINGSTATUS_STATUS_INVALID
                )
                mail_status.save()
                # Setto stato dell'EMAIL: CANCELED
                self.email.set_status(mail_status)
                transaction.commit()
                log.debug("SMTP Recipient Refused:", e)
            except (IntegrityError, DatabaseError) as er:
                transaction.rollback()
                log.debug("Database error: %s" % er.message)
            except Exception, e:
                mail_status = MailingStatus(
                    email_object=self.email,
                    diagnostic_code = e,
                    status=sent_status
                )
                mail_status.save()
                # Setto stato dell'EMAIL: CANCELED
                self.email.set_status(mail_status)
                transaction.commit()
                log.debug("SMTP error:", e)
        except Exception, e:
            transaction.rollback()
            log.debug("General error: %s" % e)
            return
        self.smtp.quit()


    def build_message(self):
        """
        Build the email as a multipart message containing
        a multipart alternative for text (plain, HTML) plus
        all the attached files.
        """
        content_html = self.build_email_content()
        content_text = html2text(content_html)

        message = MIMEMultipart()
        message['Subject'] = self.build_title_content()
        message['From'] = smart_str(self.email.header_sender)
        message['Reply-to'] = smart_str(self.email.header_reply)
        message['To'] = self.email.email_address
        message['List-ID'] = "<%s>" % self.email.id
        message['Message-Id'] = "<%s@acomea.it>" % self.email.id # from email.utils import make_msgid; make_msgid()
        message['Date'] = formatdate(localtime=True)

        message_alt = MIMEMultipart('alternative')
        message_alt.attach(MIMEText(smart_str(content_text), 'plain', 'UTF-8'))
        message_alt.attach(MIMEText(smart_str(content_html), 'html', 'UTF-8'))
        message.attach(message_alt)

        for attachment in self.attachments:
            message.attach(attachment)

        for header, value in self.email.smtp_server.custom_headers.items():
            message[header] = value

        return message

    def build_attachments(self):
        """Build email's attachment messages"""
        attachments = []

        for attachment in self.email.attachments.all():
            ctype, encoding = mimetypes.guess_type(attachment.file_attachment.path)

            if ctype is None or encoding is not None:
                ctype = 'application/octet-stream'

            maintype, subtype = ctype.split('/', 1)

            fd = open(attachment.file_attachment.path, 'rb')
            if maintype == 'text':
                message_attachment = MIMEText(fd.read(), _subtype=subtype)
            elif maintype == 'message':
                message_attachment = message_from_file(fd)
            elif maintype == 'image':
                message_attachment = MIMEImage(fd.read(), _subtype=subtype)
            elif maintype == 'audio':
                message_attachment = MIMEAudio(fd.read(), _subtype=subtype)
            else:
                message_attachment = MIMEBase(maintype, subtype)
                message_attachment.set_payload(fd.read())
                encode_base64(message_attachment)
            fd.close()

            message_attachment.add_header(
                'Content-Disposition', 'attachment',
                filename=attachment.title
            )
            attachments.append(message_attachment)

        return attachments

    def smtp_connect(self):
        """Make a connection to the SMTP"""
        self.smtp = self.email.smtp_server.connect()

    def build_title_content(self):
        """Generate the email title"""
        context = Context({
            'email': self.email,
            'UNIQUE_KEY': ''.join(sample(
                settings.MAILER_UNIQUE_KEY_CHAR_SET, settings.MAILER_UNIQUE_KEY_LENGTH
            ))
        })
        title = Template(self.email.subject).render(context)
        return title

    def build_email_content(self):
        """Generate the mail"""
        uidb36, token = tokenize(self.email)
        generic_object = self.email.content_object
        context = Context({
            'email': self.email,
            'domain': Site.objects.get_current().domain,
            'uidb36': uidb36, 'token': token,
            'generic_object': generic_object
        })

        # to use newsletter template
        # TODO: ATTENZIONE!!! newsletter2
        if self.email.content_type.app_label == 'newsletter':
            content = get_template(self.template).render(context)
        if self.email.content_type.app_label == 'newsletter2':
            content = Template(self.email.content).render(context)
        if self.email.content_type.app_label == 'event':
            content = Template(self.email.content).render(context)

        # to track link
        if settings.MAILER_TRACKING_LINKS:
            content = track_links(content, context)

        if settings.MAILER_TRACKING_LINK_SITE:
            # to insert link to view email on site

            link_site = render_to_string(
                'mailer/email_link_site.html', context
            )
            content = body_insertion(content, link_site)

        # to insert image
        if settings.MAILER_TRACKING_IMAGE:
            image_tracking = render_to_string(
                'mailer/email_image_tracking.html', context
            )
            content = body_insertion(content, image_tracking, end=True)

        # to insert unsubscribe link
        # non include unsubscribe link per applicazione eventi: forzatura richiesta da cliente
        if settings.MAILER_INCLUDE_UNSUBSCRIPTION and self.email.content_type.app_label != 'event':
            new_content = self.insert_unsuscribe_link(content, context)
            if new_content:
                content = new_content

        return smart_unicode(content)

    def insert_unsuscribe_link(self, content, context):
        from acomea.contact.utils.tokens import contact_token_generator
        from acomea.contact.models import Email
        try:
            email = Email.objects.get(email=self.email.email_address)
            contact = email.contact
            token = contact_token_generator.make_token(contact)
            cidb36 = int_to_base36(contact.id)
            context.update({
                'uidb36': cidb36,
                'token': token,
            })
        except (ObjectDoesNotExist, MultipleObjectsReturned):
            pass
        unsubscribe_link = render_to_string(
            'mailer/email_unsubscribe_link.html', context
        )
        return smart_unicode(body_insertion(content, unsubscribe_link, end=True))

    @property
    def can_send(self):
        """Check if the email can be sent"""
        if self.test:
            return True

        if self.email.smtp_server.get_credits() <= 0:
            return False

        if self.email.status == settings.MAILER_EMAIL_STATUS_TO_SEND:
            return True

        return False
