#! -*- coding: utf-8 -*-

from django.conf import settings

from jmb.mailer.models import MailingStatus


class Statistic(object):
    def __init__(self, object_id=None, content_type_id=None):
        self.object_id=object_id
        self.content_type_id=content_type_id

    def recalculate(self):
        mailing_status = MailingStatus.objects.all()
        if self.object_id:
            mailing_status = mailing_status.filter(email_object__object_id=self.object_id)
        if self.content_type_id:
            mailing_status = mailing_status.filter(email_object__content_type__id=self.content_type_id)
        mailing_status = mailing_status.order_by('date_create')

        # inizializzazione dizionari
        self.sum_per_status={}
        self.sum_per_status_per_day={}
        self.sum_delivered_per_day={}
        _unique_per_status_per_email={}
        _unique_per_link_per_email={}
        self.unique_per_status={}
        self.unique_per_status_per_day={}
        self.sum_per_link={}
        self.sum_per_link_per_day={}
        self.unique_per_link={}
        self.unique_per_link_per_day={}
        self.range=[]

        if mailing_status:
            date_begin =  mailing_status[0].date_create.date()
            date_end = mailing_status.reverse()[0].date_create.date()
            self.range = [date_begin, date_end]

        for status in mailing_status:
            date = status.date_create.strftime('%Y%m%d')
            email = status.email_object.email_address


            # sums per status
            if self.sum_per_status.get(status.status):
                self.sum_per_status[status.status] += 1
            else:
                self.sum_per_status[status.status] = 1

            # sums per status and per day
            if self.sum_per_status_per_day.get((
                status.status,
                date,
                )):
                self.sum_per_status_per_day[(
                    status.status,
                    date
                    )] += 1
            else:
                self.sum_per_status_per_day[(
                    status.status,
                    date,
                    )] = 1

            # unique
            if not _unique_per_status_per_email.get((
                status.status,
                email
                )):
                _unique_per_status_per_email[
                status.status,
                email
                ] = 1
                # unique per status
                if self.unique_per_status.get(status.status):
                    self.unique_per_status[status.status] += 1
                else:
                    self.unique_per_status[status.status] = 1

                # unique per status and per date
                if self.unique_per_status_per_day.get((
                    status.status,
                    date
                    )):
                    self.unique_per_status_per_day[(
                        status.status,
                        date
                        )] += 1
                else:
                    self.unique_per_status_per_day[(
                        status.status,
                        date
                        )] = 1

            if status.status == settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED:
                link = status.status_link.link.title
                # sums per link
                if self.sum_per_link.get(link):
                    self.sum_per_link[link] += 1
                else:
                    self.sum_per_link[link] = 1

                # sums per link per day
                if self.sum_per_link_per_day.get((
                    link, date
                    )):
                    self.sum_per_link_per_day[(
                        link, date
                        )] += 1
                else:
                    self.sum_per_link_per_day[(
                        link, date
                        )] = 1
                    # unique per link
                if not _unique_per_link_per_email.get((
                    link, email
                    )):
                    _unique_per_link_per_email[
                    link, email
                    ] = 1

                    # unique per link
                    if self.unique_per_link.get(link):
                        self.unique_per_link[link] += 1
                    else:
                        self.unique_per_link[link] = 1

                    # unique per link and per date
                    if self.unique_per_link_per_day.get((
                        link, date
                        )):
                        self.unique_per_link_per_day[(
                            link, date
                            )] += 1
                    else:
                        self.unique_per_link_per_day[(
                            link, date
                            )] = 1