"""Utils for newsletter"""
from BeautifulSoup import BeautifulSoup
from django import get_version
from django.core.urlresolvers import reverse

from jmb.mailer.models import Link


def body_insertion(content, insertion, end=False):
    """Insert an HTML content into the body HTML node"""
    if not content.startswith('<html'):
        if not content.startswith('<body'):
            content = '<body>%s</body>' % content
        content = '<html>%s</html>' % content

    soup = BeautifulSoup(content)
    html_txt = soup if get_version()>="1.4.3" else soup.body
    if end:
        html_txt.append(insertion)
    else:
        html_txt.insert(0, insertion)
    return html_txt.prettify()


def track_links(content, context):
    """Convert all links in the template for the user
    to track his navigation"""
    if not context.get('uidb36'):
        return content

    soup = BeautifulSoup(content)
    for link_markup in soup('a'):
        if (
            link_markup.get('href')
            and 'no-track' not in link_markup.get('rel', '')
            and 'mailto' not in link_markup.get('href')
        ):
            link_href = link_markup['href']
            link_title = link_markup.get('title', link_href)
            link, created = Link.objects.get_or_create(
                url=link_href,
                email_object=context['email'],
                defaults={'title': link_title}
            )
            link_markup['href'] = 'http://%s%s' % (
                context['domain'], reverse(
                    'mailer_email_tracking_link',
                    args=[context['uidb36'], context['token'],link.pk]
                )
            )
    return soup.prettify()
